<?php

namespace Drupal\gifplayer\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Implementation of Gif Player formatter.
 *
 * @FieldFormatter(
 *   id = "gif_player_video_formatter",
 *   label = @Translation("Gif Player Video"),
 *   field_types = {
 *     "gifplayer_video"
 *   }
 * )
 */
class GifPlayerVideoFormatter extends GifPlayerFormatter {

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);
    $settings = $this->getSettings();


    $elements['mode'] = [
      '#type' => 'select',
      '#title' =>$this->t('Mode'),
      '#options' => [
        'video' => 'Video',
      ],
      '#description' => $this->t(
        'Load an animated video file.'
      ),
      '#default_value' => $this->getSettings()['mode'],
      '#required' => TRUE,
      '#disabled' => TRUE,
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'label' => 'gif',
      'wait' => false,
      'playon' => 'click',
      'mode' => 'video',
      'width' => '250',
      'height' => '300',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    // Early opt-out if the field is empty.
    if (count($items) <= 0) {
      return [];
    }

    // Get the field formatter settings.
    $settings = $this->getSettings();

    $videos = $items->getEntity()->get($items->getName())->referencedEntities();
    foreach ($videos as $key => $value) {
      if ($value instanceof File) {
        $videos_links[$value->id()] =  file_create_url(
          $value->getFileUri()
        );
      }
    }

    // Load imagcover from settings if this is set.
    $cid = $this->configFactory->get('gifplayer.settings')->get('cover_png');
    $file_storage = $this->entityTypeManager->getStorage('file');

    // Set image cover path.
    if ($cid !== 0 && $file = $file_storage->load($cid)) {
      $img_cover = file_create_url($file->getFileUri());
    }
    else {
      $img_cover = file_create_url(
        drupal_get_path('module', 'gifplayer') . '/img/banana.png'
      );
    }

    // Create a renderable array.
    $output = [
      '#theme' => 'gif_player_video',
      '#gif_label' => $settings['label'],
      '#playon' => $settings['playon'],
      '#wait' => $settings['wait'],
      '#mode' => $settings['mode'],
      '#width' => $settings['width'],
      '#height' => $settings['height'],
      '#videos' => $videos_links,
      '#img_cover' => $img_cover,
    ];

    // Attach library to the field, checks if this is not added globally.
    $global_attach = $this->configFactory->get('gifplayer.settings')
      ->get('library_attach');

    if (!$global_attach) {
      $output['#attached'] = ['library' => 'gifplayer/gifplayer'];
    }

    return $output;
  }

}
